unit mainUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, Menus, db,
  dbf, Calendar, StdCtrls, ComCtrls, DbCtrls, ExtCtrls, newEntryUnit, aboutUnit,
  LazHelpHTML, HelpIntfs;

type

  { TMainForm }

  TMainForm = class(TForm)
    HelpMenu: TMenuItem;
    HelpUsageMenuItem: TMenuItem;
    AboutMenuItem: TMenuItem;
    HTMLBrowserHelpViewer1: THTMLBrowserHelpViewer;
    HTMLHelpDatabase1: THTMLHelpDatabase;
    NewEntryButton: TButton;
    EditEntryButton: TButton;
    DeleteEntryButton: TButton;
    CommentDBMemo: TDBMemo;
    MonthLabel: TLabel;
    Panel1: TPanel;
    YearLabel: TLabel;
    NotesLabel: TLabel;
    StatusBar: TStatusBar;
    TodayButton: TButton;
    Calendar: TCalendar;
    Datasource1: TDatasource;
    MyDbf: TDbf;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    OpenTableMenuItem: TMenuItem;
    CloseTableMenuItem: TMenuItem;
    MenuItem4: TMenuItem;
    ExitMenuItem: TMenuItem;
    NewTableMenuItem: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    procedure AboutMenuItemClick(Sender: TObject);
    procedure CalendarDayChanged(Sender: TObject);
    procedure CalendarMonthChanged(Sender: TObject);
    procedure CloseTableMenuItemClick(Sender: TObject);
    procedure DeleteEntryButtonClick(Sender: TObject);
    procedure EditEntryButtonClick(Sender: TObject);
    procedure ExitMenuItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpUsageMenuItemClick(Sender: TObject);
    procedure NewEntryButtonClick(Sender: TObject);
    procedure NewTableMenuItemClick(Sender: TObject);
    procedure OpenTableMenuItemClick(Sender: TObject);
    procedure TodayButtonClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    Procedure ChangeDetails;
    Procedure CreateTable(aFileName : String);
    Procedure OpenTable(aFileName : String);
    Procedure CloseTable;
    Procedure FindEntry(aDate : TDateTime);
    Procedure ActivateButtons( act : Boolean);
    stuf : String;
  end; 

var
  MainForm: TMainForm;
  myDate : TDateTime;             {This holds the current date}
  myYear, myMonth, myDay : Word;  {These hold the date broken down}


implementation

{ TMainForm }

procedure TMainForm.ExitMenuItemClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
     myDate := Date;                 {Get the current date}
     Calendar.DateTime := myDate;    { Set Calendar to the current date}
     CommentDBMemo.Clear;
     OpenTable('cal.dbf');           {Opens the 'default' table}
     StatusBar.SimpleText:= 'Database table open : cal.dbf';
     ChangeDetails;                  {Update the display to reflect the current days entry}
end;

procedure TMainForm.HelpUsageMenuItemClick(Sender: TObject);
begin
  ShowHelpOrErrorForKeyword('','HTML/index.html');
end;

{When pressed this shows the NewEntryForm dialog box and adds
the entry to the open table and displays it in the CommentBDMemo}
procedure TMainForm.NewEntryButtonClick(Sender: TObject);
begin
   NewEntryForm.Caption:= 'New Entry';
  if NewEntryForm.ShowModal = mrOk then
  begin
     stuf := NewEntryForm.stuff;
     CommentDBMemo.Lines.Add(stuf);
     MyDbf.Append;
     MyDbf.FieldByName('Comments').AsString:= stuf;
     MyDbf.FieldByName('Date').AsDateTime:= myDate;
     MyDbf.Post;
  end;
  ChangeDetails;
end;

{Creates a new table and opens it ready for entries}
procedure TMainForm.NewTableMenuItemClick(Sender: TObject);
begin
  with SaveDialog do
         if Execute then
         begin
            CreateTable(FileName);
         end;
end;

{Opens an existing table. The OpenDialogs settings insist that
the table alredy exist.}
procedure TMainForm.OpenTableMenuItemClick(Sender: TObject);
begin
   with OpenDialog do
         if Execute then
         begin
            OpenTable(FileName);
             StatusBar.SimpleText:= 'Database table open : ' + FileName;
         end;
  ActivateButtons(True);
  FindEntry(myDate);
end;

{Sets the Calendar and the CommentDBMemo display to the current date.}
procedure TMainForm.TodayButtonClick(Sender: TObject);
begin
   Calendar.DateTime:= Now;
     ChangeDetails;
end;

{If the Calendar display is clicked to show a different day then the display
and the CommentDBMemo is updated to reflect this}
procedure TMainForm.CalendarDayChanged(Sender: TObject);
begin
  ChangeDetails;
end;

{Shows the applications About form}
procedure TMainForm.AboutMenuItemClick(Sender: TObject);
begin
  AboutForm.ShowModal;
end;

{As above but the display is altered by a month}
procedure TMainForm.CalendarMonthChanged(Sender: TObject);
begin
   ChangeDetails;
end;

{Closes an open table and updates the display}
procedure TMainForm.CloseTableMenuItemClick(Sender: TObject);
begin
  CloseTable;
end;

{Deletes a table entry after asking for permission, and updates the display}
procedure TMainForm.DeleteEntryButtonClick(Sender: TObject);
begin
  if MessageDlg('Do you really want to delete this entry?', mtWarning, [mbYes, mbNo], 0) = mrYes then
   begin
   MyDbf.Edit;
   MyDbf.Delete;
   ChangeDetails;
   end;
end;

{Edits the current entry displayed. The NewEntryForm is reused and the resultant
string returned is used to update the table entry and the display}
procedure TMainForm.EditEntryButtonClick(Sender: TObject);
begin
  NewEntryForm.Caption:= 'Edit Entry';
  NewEntryForm.EntryMemo.Lines.Clear;
  NewEntryForm.EntryMemo.Lines.Add(CommentDBMemo.Lines.Text);
  if NewEntryForm.ShowModal = mrOk then
  begin
     stuf := NewEntryForm.stuff;
     CommentDBMemo.Lines.Append(stuf);
     MyDbf.Edit;
     MyDbf.FieldByName('Comments').AsString:= stuf;
     MyDbf.Post;
  end;
end;

{This updates the display and displays the current dates entry.}
procedure TMainForm.ChangeDetails;
begin
   myDate := Calendar.DateTime;
  DecodeDate(myDate, myYear, myMonth, myDay);
  MonthLabel.Caption := LongMonthNames[myMonth];
  YearLabel.Caption := IntToStr(myYear);
  NotesLabel.Caption := 'Notes for ' + LongDayNames[DayOfWeek(myDate)] + ' ' + IntToStr(myDay) + ' ' + LongMonthNames[myMonth] + ' ' + IntToStr(myYear);
  FindEntry(myDate);
end;

{Creates a new table with the name aFileName (a string),
and opens it ready for editing}
procedure TMainForm.CreateTable(aFileName: String);
begin
    with MyDbf do
     begin
          Close;
          TableName := AFileName;
          with FieldDefs do
          begin
               Clear;
               Add('Date', ftDateTime,0,False);
               Add('Comments', ftMemo,0,False);
          end;
          CreateTable;
          Exclusive := True;
          Open;
     end;
end;

{This opens an existing table (aFileName)}
procedure TMainForm.OpenTable(aFileName: String);
begin
   with MyDbf do
    begin
         Close;
         TableName := AFileName;
         Exclusive := True;
         Open;
    end;
end;

{Closes the currently open table and updates the display to reflect this }
procedure TMainForm.CloseTable;
begin
   MyDbf.Close;
  CommentDBMemo.Lines.Clear;
  StatusBar.SimpleText:= 'Database table open : NONE';
  ActivateButtons(False);
end;

{Finds an entry in the currently open table. If there is no entry for that date
then the display is updated to reflect this.}
procedure TMainForm.FindEntry(aDate: TDateTime);
var
   here : Boolean;
begin
   NewEntryButton.Enabled:= False;
  EditEntryButton.Enabled:= False;
  DeleteEntryButton.Enabled:= False;
  here := MyDbf.Locate('Date', aDate,[loCaseInsensitive ,loPartialKey]);
  if here = False then
     begin
       CommentDBMemo.Lines.Clear;
       NewEntryButton.Enabled:= True;
       EditEntryButton.Enabled:= False;
       DeleteEntryButton.Enabled:= False;
     end
     else
     begin
         NewEntryButton.Enabled:= False;
         EditEntryButton.Enabled:= True;
         DeleteEntryButton.Enabled:= True;
     end;
end;

{Enables or disables the displays buttons depending on the state of the open table}
procedure TMainForm.ActivateButtons(act: Boolean);
begin
     NewEntryButton.Enabled:= act;
     EditEntryButton.Enabled:= act;
     DeleteEntryButton.Enabled:= act;
     TodayButton.Enabled:= act;
     Calendar.Enabled:= act;
end;

initialization
  {$I mainunit.lrs}

end.

